@@ -18,12 +18,6 @@ import pkg_resources
 from py2app.apptemplate.setup import main as script_executable
 from py2app.util import mergecopy, make_exec
 
-
-try:
-    import sysconfig
-except ImportError:
-    sysconfig = None
-
 try:
     from cStringIO import StringIO
 except ImportError:
@@ -44,6 +38,8 @@ from modulegraph import zipio
 
 import macholib.dyld
 import macholib.MachOStandalone
+import macholib.MachO
+from macholib.util import flipwritable
 
 from py2app.create_appbundle import create_appbundle
 from py2app.create_pluginbundle import create_pluginbundle
@@ -59,11 +55,92 @@ from py2app import recipes
 from distutils.sysconfig import get_config_var
 PYTHONFRAMEWORK=get_config_var('PYTHONFRAMEWORK')
 
+
+PLUGIN_SUFFIXES = {
+        '.qlgenerator':    'QuickLook',
+        '.mdimporter':     'Spotlight',
+        '.xpc':            'XPCServices',
+        '.service':        'Services',
+        '.prefPane':       'PreferencePanes',
+        '.iaplugin':       'InternetAccounts',
+        '.action':         'Automator',
+}
+
 try:
     basestring
 except NameError:
     basestring = str
 
+def rewrite_tkinter_load_commands(tkinter_path):
+    print("rewrite_tk", tkinter_path)
+    m = macholib.MachO.MachO(tkinter_path)
+    tcl_path = None
+    tk_path = None
+
+    rewrite_map = {}
+
+    for header in m.headers:
+        for idx, name, other in header.walkRelocatables():
+            if other.endswith('/Tk'):
+                if tk_path is not None and other != tk_path:
+                    raise DistutilsPlatformError('_tkinter is linked to different Tk paths')
+                tk_path = other
+            elif other.endswith('/Tcl'):
+                if tcl_path is not None and other != tcl_path:
+                    raise DistutilsPlatformError('_tkinter is linked to different Tcl paths')
+                tcl_path = other
+
+    if tcl_path is None or 'Tcl.framework' not in tcl_path:
+        raise DistutilsPlatformError('_tkinter is not linked a Tcl.framework')
+
+    if tk_path is None or 'Tk.framework' not in tk_path:
+        raise DistutilsPlatformError('_tkinter is not linked a Tk.framework')
+
+    system_tcl_versions = [nm for nm in os.listdir('/System/Library/Frameworks/Tcl.framework/Versions') if nm != 'Current']
+    system_tk_versions = [nm for nm in os.listdir('/System/Library/Frameworks/Tk.framework/Versions') if nm != 'Current']
+
+    if not tcl_path.startswith('/System/Library/Frameworks'):
+        # ../Versions/8.5/Tcl
+        ver = os.path.basename(os.path.dirname(tcl_path))
+        if ver not in system_tcl_versions:
+            raise DistutilsPlatformError('_tkinter is linked to a version of Tcl not in /System')
+
+        rewrite_map[tcl_path] = '/System/Library/Frameworks/Tcl.framework/Versions/%s/Tcl'%(ver,)
+
+    if not tk_path.startswith('/System/Library/Frameworks'):
+        # ../Versions/8.5/Tk
+        ver = os.path.basename(os.path.dirname(tk_path))
+        if ver not in system_tk_versions:
+            raise DistutilsPlatformError('_tkinter is linked to a version of Tk not in /System')
+
+        rewrite_map[tk_path] = '/System/Library/Frameworks/Tk.framework/Versions/%s/Tk'%(ver,)
+
+    if rewrite_map:
+        print("Relinking _tkinter.so to system Tcl/Tk")
+        rewroteAny = False
+        for header in m.headers:
+            for idx, name, other in header.walkRelocatables():
+                data = rewrite_map.get(other)
+                if data:
+                    if header.rewriteDataForCommand(idx, data.encode(sys.getfilesystemencoding())):
+                        rewroteAny = True
+
+        if rewroteAny:
+            old_mode = flipwritable(m.filename)
+            try:
+                with open(m.filename, 'rb+') as f:
+                    for header in m.headers:
+                        f.seek(0)
+                        header.write(f)
+                        f.seek(0, 2)
+                        f.flush()
+
+            finally:
+                flipwritable(m.filename, old_mode)
+
+    else:
+        print("_tkinter already linked against system Tcl/Tk")
+
 
 def get_zipfile(dist, semi_standalone=False):
     if sys.version_info[0] == 3:
@@ -89,8 +166,8 @@ class PythonStandalone(macholib.MachOStandalone.MachOStandalone):
 
             # Ensure that the orginal name also exists, avoids problems when
             # the filename is used from Python (see issue #65)
-            # 
-            # NOTE: The if statement checks that the target link won't 
+            #
+            # NOTE: The if statement checks that the target link won't
             #       point to itself, needed for systems like homebrew that
             #       store symlinks in "public" locations that point to
             #       files of the same name in a per-package install location.
@@ -181,15 +258,21 @@ def normalize_data_file(fn):
         return ('', [fn])
     return fn
 
-def is_system(executable=None):
-    if executable is None:
-        executable = sys.executable
-    return in_system_path(executable)
+def is_system():
+    prefix = sys.prefix
+    if os.path.exists(os.path.join(prefix, ".Python")):
+        fn = os.path.join(prefix, "lib", "python%d.%d"%(sys.version_info[:2]), "orig-prefix.txt")
+        if os.path.exists(fn):
+            with open(fn, 'rU') as fp:
+                prefix = fp.read().strip()
 
-def installation_info(executable=None, version=None):
+    return in_system_path(prefix)
+
+def installation_info(version=None):
     if version is None:
         version = sys.version
-    if is_system(executable):
+
+    if is_system():
         return version[:3] + " (FORCED: Using vendor Python)"
     else:
         return version[:3]
@@ -246,13 +329,17 @@ class py2app(Command):
         ("alias", 'A',
          "Use an alias to current source file (for development only!)"),
         ("argv-emulation", 'a',
-         "Use argv emulation [disabled for plugins]. Does not work with python 3.x"),
+         "Use argv emulation [disabled for plugins]."),
         ("argv-inject=", None,
          "Inject some commands into the argv"),
         ("emulate-shell-environment", None,
          "Emulate the shell environment you get in a Terminal window"),
         ("use-pythonpath", None,
          "Allow PYTHONPATH to effect the interpreter's environment"),
+        ("use-faulthandler", None,
+         "Enable the faulthandler in the generated bundle (Python 3.3 or later)"),
+        ("verbose-interpreter", None,
+         "Start python in verbose mode"),
         ('bdist-base=', 'b',
          'base directory for build library (default is build)'),
         ('dist-dir=', 'd',
@@ -271,7 +358,9 @@ class py2app(Command):
         ("qt-plugins=", None, "set of Qt plugins to include in the application bundle (default None)"),
         ("matplotlib-backends=", None, "set of matplotlib backends to include (default: include entire package)"),
         ("extra-scripts=", None, "set of scripts to include in the application bundle, next to the main application script"),
-        ]
+        ("include-plugins=", None, "List of plugins to include"),
+        ("force-system-tk", None, "Ensure that Tkinter is linked against Apple's build of Tcl/Tk"),
+    ]
 
     boolean_options = [
         #"compressed",
@@ -284,12 +373,15 @@ class py2app(Command):
         "argv-emulation",
         #"no-zip",
         "use-pythonpath",
+        "use-faulthandler",
+        "verbose-interpreter",
         "no-chdir",
         "debug-modulegraph",
         "debug-skip-macholib",
         "graph",
         "prefer-ppc",
         "emulate-shell-environment",
+        "force-system-tk",
     ]
 
     def initialize_options (self):
@@ -314,6 +406,8 @@ class py2app(Command):
         self.no_chdir = 0
         self.site_packages = False
         self.use_pythonpath = False
+        self.use_faulthandler = False
+        self.verbose_interpreter = False
         self.includes = None
         self.packages = None
         self.excludes = None
@@ -334,6 +428,8 @@ class py2app(Command):
         self.qt_plugins = None
         self.matplotlib_backends = None
         self.extra_scripts = None
+        self.include_plugins = None
+        self.force_system_tk = False
 
     def finalize_options (self):
         if not self.strip:
@@ -345,13 +441,13 @@ class py2app(Command):
             self.argv_inject = shlex.split(self.argv_inject)
         self.includes = set(fancy_split(self.includes))
         self.includes.add('encodings.*')
+
+        if self.use_faulthandler:
+            self.includes.add('faulthandler')
         #if sys.version_info[:2] >= (3, 2):
         #    self.includes.add('pkgutil')
         #    self.includes.add('imp')
         self.packages = set(fancy_split(self.packages))
-        for pkg in self.packages:
-            if "." in pkg:
-                raise DistutilsOptionError("Cannot include subpackages using the 'packages' option")
 
         self.excludes = set(fancy_split(self.excludes))
         self.excludes.add('readline')
@@ -417,6 +513,7 @@ class py2app(Command):
         self.qt_plugins = fancy_split(self.qt_plugins)
         self.matplotlib_backends = fancy_split(self.matplotlib_backends)
         self.extra_scripts = fancy_split(self.extra_scripts)
+        self.include_plugins = fancy_split(self.include_plugins)
 
 
         if self.datamodels:
@@ -483,6 +580,7 @@ class py2app(Command):
         else:
             dylib = 'libpython%s.dylib' % (sys.version[:3],)
             runtime = os.path.join(prefix, 'lib', dylib)
+
         return dylib, runtime
 
     def symlink(self, src, dst):
@@ -499,12 +597,12 @@ class py2app(Command):
             yield runtime
 
     def run(self):
-        if sysconfig.get_config_var('PYTHONFRAMEWORK') is None:
-            if not sysconfig.get_config_var('Py_ENABLE_SHARED'):
+        if get_config_var('PYTHONFRAMEWORK') is None:
+            if not get_config_var('Py_ENABLE_SHARED'):
                 raise DistutilsPlatformError("This python does not have a shared library or framework")
 
             else:
-                # Issue .. in py2app's tracker, and issue .. in python's tracker: a unix-style shared 
+                # Issue .. in py2app's tracker, and issue .. in python's tracker: a unix-style shared
                 # library build did not read the application environment correctly. The collection of
                 # if statements below gives a clean error message when py2app is started, instead of
                 # building a bundle that will give a confusing error message when started.
@@ -589,7 +687,22 @@ class py2app(Command):
         for src, dest in self.iter_mappingmodels(resdir):
             self.mkpath(os.path.dirname(dest))
             mapc(src, dest)
-        
+
+    def iter_extra_plugins(self):
+        for item in self.include_plugins:
+            if isinstance(item, (list, tuple)):
+                subdir, path = item
+
+            else:
+                ext = os.path.splitext(item)[1]
+                try:
+                    subdir = PLUGIN_SUFFIXES[ext]
+                    path = item
+                except KeyError:
+                    raise DistutilsOptionError("Cannot determine subdirectory for plugin %s"%(item,))
+
+            yield path, os.path.join(subdir, os.path.basename(path))
+
     def iter_data_files(self):
         dist = self.distribution
         allres = chain(getattr(dist, 'data_files', ()) or (), self.resources)
@@ -624,13 +737,15 @@ class py2app(Command):
                 emulate_shell_environment=bool(self.emulate_shell_environment),
                 no_chdir=bool(self.no_chdir),
                 prefer_ppc=self.prefer_ppc,
+                verbose=self.verbose_interpreter,
+                use_faulthandler=self.use_faulthandler,
             ),
         )
         if self.optimize:
             result['PyOptions']['optimize'] = self.optimize
         return result
 
-    
+
     def initialize_plist(self):
         plist = self.get_default_plist()
         for target in self.targets:
@@ -670,7 +785,7 @@ class py2app(Command):
                 elif fn.endswith('.pyw'):
                     fn = fn[:-4]
 
-                src_fn = script_executable(arch=self.arch)
+                src_fn = script_executable(arch=self.arch, secondary=True)
                 tgt_fn = os.path.join(target.appdir, 'Contents', 'MacOS', os.path.basename(fn))
                 mergecopy(src_fn, tgt_fn)
                 make_exec(tgt_fn)
@@ -715,8 +830,8 @@ class py2app(Command):
                     flatpackages[pkg[0]] = pkg[1]
                 filters.extend(rval.get('filters', ()))
                 loader_files.extend(rval.get('loader_files', ()))
-                newbootstraps = map(self.get_bootstrap,
-                    rval.get('prescripts', ()))
+                newbootstraps = list(map(self.get_bootstrap,
+                    rval.get('prescripts', ())))
 
                 if rval.get('includes'):
                     find_needed_modules(mf, includes=rval['includes'])
@@ -727,6 +842,7 @@ class py2app(Command):
                 for fn in newbootstraps:
                     if isinstance(fn, basestring):
                         mf.run_script(fn)
+
                 for target in self.targets:
                     target.prescripts.extend(newbootstraps)
                 break
@@ -760,8 +876,7 @@ class py2app(Command):
 
     def build_xref(self, mf, flatpackages):
         for target in self.targets:
-            base = target.get_dest_base()
-            appdir = os.path.join(self.dist_dir, os.path.dirname(base))
+            appdir = target.appdir
             appname = self.get_appname()
             dgraph = os.path.join(appdir, appname + '.html')
             print("*** creating dependency html: %s ***"
@@ -783,15 +898,16 @@ class py2app(Command):
     def finalize_modulefinder(self, mf):
         for item in mf.flatten():
             if isinstance(item, Package) and item.filename == '-':
-                fn = os.path.join(self.temp_dir, 'empty_package', '__init__.py')
-                if not os.path.exists(fn):
-                    dn = os.path.dirname(fn)
-                    if not os.path.exists(dn):
-                        os.makedirs(dn)
-                    with open(fn, 'w') as fp:
-                        pass
+                if sys.version_info[:2] <= (3,3):
+                    fn = os.path.join(self.temp_dir, 'empty_package', '__init__.py')
+                    if not os.path.exists(fn):
+                        dn = os.path.dirname(fn)
+                        if not os.path.exists(dn):
+                            os.makedirs(dn)
+                        with open(fn, 'w') as fp:
+                            pass
 
-                item.filename = fn
+                    item.filename = fn
 
         py_files, extensions = parse_mf_results(mf)
 
@@ -841,7 +957,7 @@ class py2app(Command):
 
         if os.path.exists(self.bdist_dir):
             shutil.rmtree(self.bdist_dir)
-        
+
         self.collect_dir = os.path.abspath(
             os.path.join(self.bdist_dir, "collect"))
         self.mkpath(self.collect_dir)
@@ -960,7 +1076,20 @@ class py2app(Command):
 
                 else:
                     self.copy_file(sys.executable, execdst)
+
             if not self.debug_skip_macholib:
+                if self.force_system_tk:
+                    print("force system tk")
+                    resdir = os.path.join(dst, 'Contents', 'Resources')
+                    pydir = os.path.join(resdir, 'lib', 'python%s.%s'%(sys.version_info[:2]))
+                    ext_dir = os.path.join(pydir, os.path.basename(self.ext_dir))
+                    tkinter_path = os.path.join(ext_dir, '_tkinter.so')
+                    if os.path.exists(tkinter_path):
+                        rewrite_tkinter_load_commands(tkinter_path)
+                    else:
+                        print("tkinter not found at", tkinter_path)
+
+
                 mm = PythonStandalone(self, dst, executable_path=exp)
                 dylib, runtime = self.get_runtime()
                 if self.semi_standalone:
@@ -1131,11 +1260,8 @@ class py2app(Command):
         # XXX - In this particular case we know exactly what we can
         #       get away with.. should this be extended to the general
         #       case?  Per-framework recipes?
-        includedir = None
-        configdir = None
-        if sysconfig is not None:
-            includedir = sysconfig.get_config_var('CONFINCLUDEPY')
-            configdir = sysconfig.get_config_var('LIBPL')
+        includedir = get_config_var('CONFINCLUDEPY')
+        configdir = get_config_var('LIBPL')
 
 
         if includedir is None:
@@ -1160,6 +1286,15 @@ class py2app(Command):
                 os.path.basename(outdir),
                 os.path.join(os.path.dirname(outdir), "Current"))
 
+        # Likewise for two links in the root of the framework:
+        os.symlink(
+            'Versions/Current/Resources',
+            os.path.join(os.path.dirname(os.path.dirname(outdir)), 'Resources'))
+        os.symlink(
+            os.path.join('Versions/Current', PYTHONFRAMEWORK),
+            os.path.join(os.path.dirname(os.path.dirname(outdir)), PYTHONFRAMEWORK))
+
+
         # Experiment for issue 57
         if not os.path.exists(os.path.join(indir, 'include')):
             alt = os.path.join(indir, 'Versions/Current')
@@ -1179,8 +1314,12 @@ class py2app(Command):
             os.path.basename(info['name']),
             'Resources/Info.plist',
             'include/%s/pyconfig.h'%(includedir),
-            'lib/%s/%s/Makefile'%(pydir, configdir)
         ]
+        if '_sysconfigdata' not in sys.modules:
+            fwkfiles.append(
+               'lib/%s/%s/Makefile'%(pydir, configdir)
+            )
+
         for fn in fmwkfiles:
             self.copy_file(
                 os.path.join(indir, fn),
@@ -1282,6 +1421,12 @@ class py2app(Command):
         #if self.style == 'app':
         #    prescripts.append('setup_pkgresource')
 
+        included_subpkg = [pkg for pkg in self.packages if '.' in pkg]
+        if included_subpkg:
+            prescripts.append('setup_included_subpackages')
+            prescripts.append(StringIO('_path_hooks = %r'%(
+                included_subpkg)))
+
         if self.emulate_shell_environment:
             prescripts.append('emulate_shell_environment')
 
@@ -1400,7 +1545,7 @@ class py2app(Command):
             else:
                 basename = fmwk['shortname'] + '.framework'
                 yield os.path.join(fmwk['location'], basename)
-    
+
     def build_alias_executable(self, target, script, extra_scripts):
         # Build an alias executable for the target
         appdir, resdir, plist = self.create_bundle(target, script)
@@ -1425,8 +1570,8 @@ class py2app(Command):
         self.symlink(
             os.path.join(realhome, 'config'),
             os.path.join(pyhome, 'config'))
-            
-        
+
+
         # symlink data files
         # XXX: fixme: need to integrate automatic data conversion
         for src, dest in self.iter_data_files():
@@ -1441,6 +1586,20 @@ class py2app(Command):
                 traceback.print_exc()
                 raise
 
+        plugindir = os.path.join(appdir, 'Contents', 'Library')
+        for src, dest in self.iter_extra_plugins():
+            dest = os.path.join(plugindir, dest)
+            if src == dest:
+                continue
+
+            makedirs(os.path.dirname(dest))
+            try:
+                copy_resource(src, dest, dry_run=self.dry_run)
+            except:
+                import traceback
+                traceback.print_exc()
+                raise
+
         # symlink frameworks
         for src in self.iter_frameworks():
             dest = os.path.join(
@@ -1480,7 +1639,6 @@ class py2app(Command):
         target.appdir = appdir
         return appdir
 
-
     def build_executable(self, target, arcname, pkgexts, copyexts, script, extra_scripts):
         # Build an executable for the target
         appdir, resdir, plist = self.create_bundle(target, script)
@@ -1494,7 +1652,7 @@ class py2app(Command):
             elif fn.endswith('.pyw'):
                 fn = fn[:-4]
 
-            src_fn = script_executable(arch=self.arch)
+            src_fn = script_executable(arch=self.arch, secondary=True)
             tgt_fn = os.path.join(self.appdir, 'Contents', 'MacOS', os.path.basename(fn))
             mergecopy(src_fn, tgt_fn)
             make_exec(tgt_fn)
@@ -1513,147 +1671,8 @@ class py2app(Command):
             os.unlink(site_path)
 
 
-        includedir = None
-        configdir = None
-        if sysconfig is not None:
-            includedir = sysconfig.get_config_var('CONFINCLUDEPY')
-            configdir = sysconfig.get_config_var('LIBPL')
-
-
-        if includedir is None:
-            includedir = 'python%d.%d'%(sys.version_info[:2])
-        else:
-            includedir = os.path.basename(includedir)
-
-        if configdir is None:
-            configdir = 'config'
-        else:
-            configdir = os.path.basename(configdir)
-
-        self.compile_datamodels(resdir)
-        self.compile_mappingmodels(resdir)
-
-        bootfn = '__boot__'
-        bootfile = open(os.path.join(resdir, bootfn + '.py'), 'w')
-        for fn in target.prescripts:
-            bootfile.write(self.get_bootstrap_data(fn))
-            bootfile.write('\n\n')
-
-        bootfile.write("DEFAULT_SCRIPT=%r\n"%(os.path.basename(script),))
-        bootfile.write('try:\n')
-        bootfile.write('    _run()\n' % os.path.realpath(script))
-        bootfile.write('except KeyboardInterrupt:\n')
-        bootfile.write('    pass\n')
-        bootfile.close()
-
-        target.appdir = appdir
-        return appdir
-
-    def build_executable(self, target, arcname, pkgexts, copyexts, script, extra_scripts):
-        # Build an executable for the target
-        appdir, resdir, plist = self.create_bundle(target, script)
-        self.appdir = appdir
-        self.resdir = resdir
-        self.plist = plist
-
-        for fn in extra_scripts:
-            if fn.endswith('.py'):
-                fn = fn[:-3]
-            elif fn.endswith('.pyw'):
-                fn = fn[:-4]
-
-            src_fn = script_executable(arch=self.arch)
-            tgt_fn = os.path.join(self.appdir, 'Contents', 'MacOS', os.path.basename(fn))
-            mergecopy(src_fn, tgt_fn)
-            make_exec(tgt_fn)
-
-
-        site_path = os.path.join(resdir, 'site.py')
-        byte_compile([
-            SourceModule('site', site_path),
-            ],
-            target_dir=resdir,
-            optimize=self.optimize,
-            force=self.force,
-            verbose=self.verbose,
-            dry_run=self.dry_run)
-        if not self.dry_run:
-            os.unlink(site_path)
-
-
-        includedir = None
-        configdir = None
-        if sysconfig is not None:
-            includedir = sysconfig.get_config_var('CONFINCLUDEPY')
-            configdir = sysconfig.get_config_var('LIBPL')
-
-
-        if includedir is None:
-            includedir = 'python%d.%d'%(sys.version_info[:2])
-        else:
-            includedir = os.path.basename(includedir)
-
-        if configdir is None:
-            configdir = 'config'
-        else:
-            configdir = os.path.basename(configdir)
-
-        self.compile_datamodels(resdir)
-        self.compile_mappingmodels(resdir)
-
-        bootfn = '__boot__'
-        bootfile = open(os.path.join(resdir, bootfn + '.py'), 'w')
-        for fn in target.prescripts:
-            bootfile.write(self.get_bootstrap_data(fn))
-            bootfile.write('\n\n')
-
-        bootfile.write("DEFAULT_SCRIPT=%r\n"%(os.path.basename(script),))
-        bootfile.write('try:\n')
-        bootfile.write('    _run()\n' % os.path.realpath(script))
-        bootfile.write('except KeyboardInterrupt:\n')
-        bootfile.write('    pass\n')
-        bootfile.close()
-
-        target.appdir = appdir
-        return appdir
-
-    def build_executable(self, target, arcname, pkgexts, copyexts, script, extra_scripts):
-        # Build an executable for the target
-        appdir, resdir, plist = self.create_bundle(target, script)
-        self.appdir = appdir
-        self.resdir = resdir
-        self.plist = plist
-
-        for fn in extra_scripts:
-            if fn.endswith('.py'):
-                fn = fn[:-3]
-            elif fn.endswith('.pyw'):
-                fn = fn[:-4]
-
-            src_fn = script_executable(arch=self.arch)
-            tgt_fn = os.path.join(self.appdir, 'Contents', 'MacOS', os.path.basename(fn))
-            mergecopy(src_fn, tgt_fn)
-            make_exec(tgt_fn)
-
-
-        site_path = os.path.join(resdir, 'site.py')
-        byte_compile([
-            SourceModule('site', site_path),
-            ],
-            target_dir=resdir,
-            optimize=self.optimize,
-            force=self.force,
-            verbose=self.verbose,
-            dry_run=self.dry_run)
-        if not self.dry_run:
-            os.unlink(site_path)
-
-
-        includedir = None
-        configdir = None
-        if sysconfig is not None:
-            includedir = sysconfig.get_config_var('CONFINCLUDEPY')
-            configdir = sysconfig.get_config_var('LIBPL')
+        includedir = get_config_var('CONFINCLUDEPY')
+        configdir = get_config_var('LIBPL')
 
 
         if includedir is None:
@@ -1703,6 +1722,13 @@ class py2app(Command):
         realhome = os.path.join(sys.prefix, 'lib', 'python' + sys.version[:3])
         self.mkpath(pydir)
 
+        # The site.py file needs to be a two locations
+        # 1) in lib/pythonX.Y, to be found during normal startup and
+        #    by the 'python' executable
+        # 2) in the resources directory next to the script for
+        #    semistandalone builds (the lib/pythonX.Y directory is too
+        #    late on sys.path to be found in that case).
+        #
         if self.optimize:
             self.symlink('../../site.pyo', os.path.join(pydir, 'site.pyo'))
         else:
@@ -1715,10 +1741,15 @@ class py2app(Command):
             self.symlink(real_include, os.path.join(resdir, 'include'))
         else:
             self.mkpath(cfgdir)
-            for fn in 'Makefile', 'Setup', 'Setup.local', 'Setup.config':
-                rfn = os.path.join(realcfg, fn)
-                if os.path.exists(rfn):
-                    self.copy_file(rfn, os.path.join(cfgdir, fn))
+            if '_sysconfigdata' not in sys.modules:
+                # Recent enough versions of Python 2.7 and 3.x have
+                # an _sysconfigdata module and don't need the Makefile
+                # to provide the sysconfig data interface. Don't copy
+                # them.
+                for fn in 'Makefile', 'Setup', 'Setup.local', 'Setup.config':
+                    rfn = os.path.join(realcfg, fn)
+                    if os.path.exists(rfn):
+                       self.copy_file(rfn, os.path.join(cfgdir, fn))
 
             inc_dir = os.path.join(resdir, 'include', includedir)
             self.mkpath(inc_dir)
@@ -1730,17 +1761,31 @@ class py2app(Command):
         if sys.version_info[0] != 2:
             import zlib
             self.copy_file(zlib.__file__, os.path.dirname(arcdir))
-        
+
         ext_dir = os.path.join(pydir, os.path.basename(self.ext_dir))
         self.copy_tree(self.ext_dir, ext_dir, preserve_symlinks=True)
         self.copy_tree(self.framework_dir,
             os.path.join(appdir, 'Contents', 'Frameworks'),
             preserve_symlinks=True)
-        for pkg in self.packages:
-            pkg = self.get_bootstrap(pkg)
-            dst = os.path.join(pydir, os.path.basename(pkg))
+        for pkg_name in self.packages:
+            pkg = self.get_bootstrap(pkg_name)
+            print('XXXX', pkg_name, pkg)
+
+            if self.semi_standalone:
+                # For semi-standalone builds don't copy packages
+                # from the stdlib into the app bundle, even when
+                # they are mentioned in self.packages.
+                p = Package(pkg_name, pkg)
+                if not not_stdlib_filter(p):
+                    continue
+
+            dst = os.path.join(pydir, pkg_name)
             self.mkpath(dst)
             self.copy_tree(pkg, dst)
+
+            # FIXME: The python files should be bytecompiled
+            #        here (see issue 101)
+
         for copyext in copyexts:
             fn = os.path.join(ext_dir,
                 (copyext.identifier.replace('.', os.sep) +
@@ -1749,34 +1794,6 @@ class py2app(Command):
             self.mkpath(os.path.dirname(fn))
             copy_file(copyext.filename, fn, dry_run=self.dry_run)
 
-        if 0 and sys.version_info[:2] >= (3, 2) and not self.alias:
-            import encodings
-            import encodings.cp437
-            import encodings.utf_8
-            import encodings.latin_1
-            import codecs
-
-            encodings_dir = os.path.join(pydir, 'encodings')
-            self.mkpath(encodings_dir)
-
-            byte_compile([
-                    SourceModule('encodings.__init__', encodings.__file__),
-                    SourceModule('encodings.cp437', encodings.cp437.__file__),
-                    SourceModule('encodings.utf_8', encodings.utf_8.__file__),
-                    SourceModule('encodings.latin_1', encodings.latin_1.__file__),
-                    SourceModule('codecs', codecs.__file__),
-                ],
-                target_dir=pydir,
-                optimize=self.optimize,
-                force=self.force,
-                verbose=self.verbose,
-                dry_run=self.dry_run)
-
-            if not self.dry_run:
-                fp = open(os.path.join(encodings_dir, 'aliases.py'), 'w')
-                fp.write('aliases = {}\n')
-                fp.close()
-
         for src, dest in self.iter_data_files():
             dest = os.path.join(resdir, dest)
             if src == dest:
@@ -1784,6 +1801,15 @@ class py2app(Command):
             makedirs(os.path.dirname(dest))
             copy_resource(src, dest, dry_run=self.dry_run)
 
+        plugindir = os.path.join(appdir, 'Contents', 'Library')
+        for src, dest in self.iter_extra_plugins():
+            dest = os.path.join(plugindir, dest)
+            if src == dest:
+                continue
+
+            makedirs(os.path.dirname(dest))
+            copy_resource(src, dest, dry_run=self.dry_run)
+
 
         target.appdir = appdir
         return appdir
@@ -1831,10 +1857,21 @@ class py2app(Command):
             save_cwd = os.getcwd()
             os.chdir(base_dir)
             for dirpath, dirnames, filenames in os.walk('.'):
+                if filenames:
+                    # Ensure that there are directory entries for
+                    # all directories in the zipfile. This is a
+                    # workaround for <http://bugs.python.org/issue14905>:
+                    # zipimport won't consider 'pkg/foo.py' to be in
+                    # namespace package 'pkg' unless there is an
+                    # entry for the directory (or there is a
+                    # pkg/__init__.py file as well)
+                    z.write(dirpath, dirpath)
+
                 for fn in filenames:
                     path = os.path.normpath(os.path.join(dirpath, fn))
                     if os.path.isfile(path):
                         z.write(path, path)
+
             os.chdir(save_cwd)
             z.close()
 
