@@ -1,6 +1,6 @@
 from __future__ import print_function
 
-import os, sys, imp, time, errno
+import os, sys, imp, time, errno, stat
 from modulegraph.find_modules import PY_SUFFIXES, C_SUFFIXES
 from modulegraph.util import *
 from modulegraph import zipio
@@ -59,7 +59,7 @@ def path_to_zip(path):
             raise DistutilsFileError("File doesn't exist: %s"%(orig_path,))
 
         try:
-           zf = zipfile.ZipFile(path)
+            zf = zipfile.ZipFile(path)
         except zipfile.BadZipfile:
             raise DistutilsFileError("File doesn't exist: %s"%(orig_path,))
 
@@ -117,7 +117,7 @@ def copy_resource(source, destination, dry_run=0, symlink=0):
             if not os.path.exists(destination):
                 os.mkdir(destination)
         for fn in zipio.listdir(source):
-            copy_resource(os.path.join(source, fn), 
+            copy_resource(os.path.join(source, fn),
                     os.path.join(destination, fn), dry_run=dry_run, symlink=symlink)
 
     else:
@@ -128,7 +128,7 @@ def copy_resource(source, destination, dry_run=0, symlink=0):
                 os.symlink(os.path.abspath(source), destination)
 
         else:
-            copy_file(source, destination, dry_run=dry_run)
+            copy_file(source, destination, dry_run=dry_run, preserve_mode=True)
 
 
 
@@ -153,12 +153,19 @@ def _copy_file(source, destination, preserve_mode=False, preserve_times=False, u
                 fp_out.write(data)
 
             if preserve_mode:
-                # XXX: copy current mode
-                pass
+                mode = None
+                if hasattr(zipio, 'getmode'):
+                    mode = zipio.getmode(source)
+
+                elif os.path.isfile(source):
+                    mode = os.stat(source).st_mode
+
+                if mode is not None:
+                    os.chmod(destination, mode)
 
             if preserve_times:
-                # XXX: copy current times
-                pass
+                mtime = zipio.getmtime(source)
+                os.utime(destination, (mtime, mtime))
 
 
 def newer(source, target):
@@ -373,7 +380,7 @@ byte_compile(files, optimize=%r, force=%r,
             if force or newer(mod.filename, cfile):
                 if verbose:
                     print("byte-compiling %s to %s" % (mod.filename, dfile))
-                    
+
                 if not dry_run:
                     mkpath(os.path.dirname(cfile))
                     suffix = os.path.splitext(mod.filename)[1]
@@ -392,14 +399,14 @@ byte_compile(files, optimize=%r, force=%r,
                         # Minor problem: This will happily copy a file
                         # <mod>.pyo to <mod>.pyc or <mod>.pyc to
                         # <mod>.pyo, but it does seem to work.
-                        copy_file(mod.filename, cfile)
+                        copy_file(mod.filename, cfile, preserve_times=True)
 
                     else:
                         raise RuntimeError \
                               ("Don't know how to handle %r" % mod.filename)
             else:
                 if verbose:
-                    print("skipping byte-compilation of %s to %s" % 
+                    print("skipping byte-compilation of %s to %s" %
                           (mod.filename, dfile))
 
 SCMDIRS = ['CVS', '.svn', '.hg', '.git']
@@ -593,7 +600,7 @@ def _get_tool(toolname):
     if toolname not in _tools:
         if os.path.exists('/usr/bin/xcrun'):
             try:
-                _tools[toolname] = check_output(['/usr/bin/xcrun', '-find', 'momc'])[:-1]
+                _tools[toolname] = check_output(['/usr/bin/xcrun', '-find', toolname])[:-1]
             except subprocess.CalledProcessError:
                 raise IOError("Tool %r not found"%(toolname,))
 
@@ -601,7 +608,7 @@ def _get_tool(toolname):
             # Support for Xcode 3.x and earlier
             if toolname == 'momc':
                 choices = [
-    	            '/Library/Application Support/Apple/Developer Tools/Plug-ins/XDCoreDataModel.xdplugin/Contents/Resources/momc',
+                    '/Library/Application Support/Apple/Developer Tools/Plug-ins/XDCoreDataModel.xdplugin/Contents/Resources/momc',
                     '/Developer/Library/Xcode/Plug-ins/XDCoreDataModel.xdplugin/Contents/Resources/momc',
                     '/Developer/usr/bin/momc',
                 ]
@@ -616,9 +623,10 @@ def _get_tool(toolname):
             for fn in choices:
                 if os.path.exists(fn):
                     _tools[toolname] = fn
-                    break 
+                    break
             else:
                 raise IOError("Tool %r not found"%(toolname,))
+    return _tools[toolname]
 
 
 def momc(src, dst):
